///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPATTACH.CPP
//
//  Description
//      This file implements the helper function of the CXPLogon class for
//      writing and retrieving attachments.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
#include "XPCUST.H"

#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

static const SizedSPropTagArray (2, sptAttachmentNum) = 
{
    2,
    {
        PR_ATTACH_NUM,
        PR_ATTACH_METHOD
    }
};

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::ReadAttachsFromFile ()
//
//    Parameters
//      pMsgObj     Pointer to message object onto where we will insert
//                  the attachments
//      pRootObj    Pointer to the storage object where the attachments
//                  are stored during a transmission
//
//    Purpose
//      This method opens and retrieves the attachments data streams.
//      The attachment table is read from the message object. The
//      other attachment properties where encoded in the TNEF stream. And
//      based upon the attachment method we read the data from the attachment
//      data file or not
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::ReadAttachsFromFile (LPMESSAGE pMsgObj, LPSTORAGE pRootObj)
{
    InfoTrace ("CXPLogon::ReadAttachsFromFile method called");
    
    // Check for attachments in message
    LPMAPITABLE pTable;
    HRESULT hResult = pMsgObj->GetAttachmentTable (0, &pTable);
    if (hResult)
    {
        TraceResult ("CXPLogon::ReadAttachsFromFile: Failed to open attachment table", hResult);
        return hResult;
    }
    SPropValue spvAttMethod = { 0 };
    spvAttMethod.ulPropTag = PR_ATTACH_METHOD;
    spvAttMethod.Value.l = ATTACH_BY_VALUE;

    SRestriction srMethod = { 0 };
    srMethod.rt = RES_PROPERTY;
    srMethod.res.resProperty.relop = RELOP_EQ;
    srMethod.res.resProperty.ulPropTag = PR_ATTACH_METHOD;
    srMethod.res.resProperty.lpProp = &spvAttMethod;

    LPSRowSet pRows = NULL;
    hResult = HrQueryAllRows (pTable,
                              (LPSPropTagArray)&sptAttachmentNum,
                              &srMethod,
                              NULL,
                              0,
                              &pRows);
    if (hResult)
    {
        TraceResult ("CXPLogon::ReadAttachsFromFile: Failed to get the rows from the attachment table", hResult);
        pTable->Release();
        return hResult;
    }

    LPATTACH pAttach = NULL;
    LPSPropValue pProps;
    ULONG ulCounter = 1; // This counter is used to create the stream name
    for (ULONG i=0; i<pRows->cRows && !hResult; i++)
    {
        // For ease of coding
        pProps = pRows->aRow[i].lpProps;
        
        // The attachment number is a "Magic" number used by MAPI to 
		// access the attachment. It is only valid in the context of
		// the current message. Do not transmit this number
		hResult = pMsgObj->OpenAttach (pProps->Value.l,
                                       &IID_IMAPIProp,
                                       MAPI_MODIFY,
                                       &pAttach);
        TraceResult ("CXPLogon::ReadAttachsFromFile: Failed to open attachment", hResult);
		if (!hResult)
		{
			hResult = GetAttachStream (pRootObj, pAttach, ulCounter);
            if (!hResult)
            {
                // Now we need to save changes on the message
                pAttach->SaveChanges (0);
            }
            // Release the message attachment object
            pAttach->Release();
            pAttach = NULL;
            // Increase counter so we can get the next available stream
            ulCounter++;
        }
    }
    // Clean up memory for the row set
    FreeProws (pRows);
    // Release used objects
    if (pAttach)
    {
        pAttach->Release();
    }
    pTable->Release();
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::GetAttachStream
//
//    Parameters
//      pRootObj    Pointer to an IStorage object. This is storage is the root
//                  in the message file where the message is stored
//      pAttachObj  Pointer to an IAttach object where the attachment is 
//                  copied into
//      ulCounter   Attachment counter number. This counter is internal to 
//                  this transport. Counts how many attachments have we found.
//                  This number is used to create a stream name where the 
//                  attachments properties are read from
//
//    Purpose
//      This method opens a IStream in the message file (pRootObj) and 
//      reads its contents into the IAttach object.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::GetAttachStream (LPSTORAGE      pRootObj,
                                        LPATTACH       pAttachObj,
                                        ULONG          ulCounter)
{
    InfoTrace ("CXPLogon::GetAttachStream method called");
    
    OLECHAR wsName[32];
    LPSTREAM pStreamObj, pAttachStreamObj;
    
    // OLE 2.0 32-bit is UNICODE internally, unless we #define OLE2ANSI and 
    // use the wrappers in OLE2ANSI.LIB. For efficiency in this transport we
    // will use native UNICODE
    wsprintfW (wsName, ATTACH_ITEM_NAME, ulCounter);
    HRESULT hResult = pRootObj->OpenStream (wsName,
                                            NULL,
                                            XP_OPEN_FLAGS,
                                            NULL,
                                            &pStreamObj);
    if (hResult)
    {
        TraceResult ("CXPLogon::GetAttachStream: Failed to open attached stream", hResult);
        return hResult;
    }
    
    // Open an IStream interface on the message attachment
    hResult = pAttachObj->OpenProperty (PR_ATTACH_DATA_BIN,
										&IID_IStream,
										0,
										MAPI_CREATE | MAPI_MODIFY,
										(LPUNKNOWN *)&pAttachStreamObj);
    if (hResult)
	{	
        TraceResult ("CXPLogon::GetAttachStream: Could not open an IStream in the message attachment", hResult);
        // Cleanup
        pStreamObj->Release();
        return hResult;
    }
    
    STATSTG StatInfo;
    hResult = pStreamObj->Stat (&StatInfo, STATFLAG_NONAME);
    if (hResult)
    {
        TraceResult ("CXPLogon::GetAttachStream: Failed to get stat info on the stream object", hResult);
        goto GetAttachStreamFinished;
    }
    
    // Copy the entire attached stream to the message attachment
    hResult = pStreamObj->CopyTo (pAttachStreamObj, StatInfo.cbSize, NULL, NULL);
    if (hResult)
    {
        TraceResult ("CXPLogon::GetAttachStream: Failed to copy attached stream", hResult);
        goto GetAttachStreamFinished;
    }

    // Commit the copied data before releasing the stream
    hResult = pAttachStreamObj->Commit (STGC_DEFAULT);
    if (hResult)
    {
        TraceResult ("CXPLogon::GetAttachStream: Failed to commit data onto attachment", hResult);
        goto GetAttachStreamFinished;
    }

GetAttachStreamFinished :
    // Clean up
    pAttachStreamObj->Release();
    pStreamObj->Release();
    
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::WriteAttachsToFile()
//
//    Parameters
//      pMsgObj     Pointer to an IMessage object. This is the actual message
//                  we are trying to send, as pass to us by the MAPI spooler
//      dwFileID    ID for create a unique file name for the attachment
//
//    Purpose
//      The function checks the attachment table and save them if any. The 
//      message attachments are save into a separate data file
//      containing the information in the PR_ATTACH_DATA_BIN or
//      PR_ATTACH_DATA_OBJ properties of the attachments. The other
//      attachments properties are encoded in the TNEF data stream
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::WriteAttachsToFile (LPMESSAGE pMsgObj, DWORD dwFileID)
{
    InfoTrace ("CXPLogon::WriteAttachsToFile method called");
    
    // MUST initialize pointers to NULL.
    LPSRowSet       pRows            = NULL;
    LPSTORAGE       pRootStg         = NULL;
    LPSTREAM        pSrcStream       = NULL;
    LPATTACH        pAttach          = NULL;
    LPSTREAM        pDstStream       = NULL;

    // Check for attachments in message
    LPMAPITABLE pTable;
    HRESULT hResult = pMsgObj->GetAttachmentTable (0, &pTable);
    if (hResult)
    {
        TraceResult ("CXPLogon::WriteAttachsToFile: Failed to open attachment table", hResult);
        return hResult;
    }

    SPropValue spvAttMethod = { 0 };
    spvAttMethod.ulPropTag = PR_ATTACH_METHOD;
    spvAttMethod.Value.l = ATTACH_BY_VALUE;

    SRestriction srMethod = { 0 };
    srMethod.rt = RES_PROPERTY;
    srMethod.res.resProperty.relop = RELOP_EQ;
    srMethod.res.resProperty.ulPropTag = PR_ATTACH_METHOD;
    srMethod.res.resProperty.lpProp = &spvAttMethod;

    hResult = HrQueryAllRows (pTable,
                              (LPSPropTagArray)&sptAttachmentNum,
                              &srMethod,
                              NULL,
                              0,
                              &pRows);
    if (hResult)
    {
        TraceResult ("CXPLogon::WriteAttachsToFile: Failed to get the rows from the attachment table", hResult);
        pTable->Release();
        return hResult;
    }
    
    OLECHAR wsName[32], wsFile[MAX_PATH];
    LPSPropValue pProps;
    wsprintfW (wsFile, W_ATT_FILENAME_FORMAT, W_ATT_FILENAME_PREFIX, dwFileID, W_ATT_FILENAME_SUFFIX);
    
    for (ULONG i=0; i<pRows->cRows; i++)
    {
        // Create the file (a compound document) when we can create storages
        // and stream for the attachments
        if (NULL == pRootStg)
        {
            hResult = StgCreateDocfile (wsFile, XP_CREATE_FLAGS, 0, &pRootStg);
            if (hResult)
            {
                TraceResult ("CXPLogon::WriteAttachsToFile: Failed to create storage object", hResult);
                goto WriteAttachsToFileFinished;
            }
        }

        // Traverse all the rows and open each attachment. pRows->cRows
        // is the number of attachments to this message
        
        // For ease of coding
        pProps = pRows->aRow[i].lpProps;
        ASSERTMSG (pProps, "Where is the property array?");
        ASSERTMSG (PR_ATTACH_NUM == pProps[0].ulPropTag, "Column 0 != PR_ATTACH_NUM");
        ASSERTMSG (PR_ATTACH_METHOD == pProps[1].ulPropTag, "Column 1 != PR_ATTACH_METHOD");
		
        // The attachment number is a "Magic" number used by the service provider to 
		// access the attachment. It is only valid in the context of
		// the current message. Do not transmit this number
		hResult = pMsgObj->OpenAttach (pProps[0].Value.l,
                                       &IID_IMAPIProp,
                                       0,
                                       &pAttach);
		if (hResult)
		{
			TraceResult ("CXPLogon::WriteAttachsToFile: Failed to open attachment", hResult);
			goto WriteAttachsToFileFinished;
		}
        
        ASSERTMSG (ATTACH_BY_VALUE == pProps[1].Value.l, "I'm not supposed to handle this attachment type!!!");
        hResult = pAttach->OpenProperty (PR_ATTACH_DATA_BIN,
                    	                 &IID_IStream,
                    					 0,
                    					 0,
                    					 (LPUNKNOWN *)&pSrcStream);
        if (hResult)
        {
            // If we can't open the interface, fail the message submission
        	TraceResult ("CXPLogon::WriteAttachsToFile: Could not open an IStream interface on the attachment", hResult);
        	goto WriteAttachsToFileFinished;
        }
        wsprintfW (wsName, ATTACH_ITEM_NAME, i+1);
		hResult = pRootStg->CreateStream (wsName,
                                          XP_CREATE_FLAGS,
                                          0,
                                          0,
                                          &pDstStream);
		if (hResult)
		{
			TraceResult ("CXPLogon::WriteAttachsToFile: Failed to open a stream on the destination storage", hResult);
			goto WriteAttachsToFileFinished;
		}
        // Get the size of the stream of the attached object
		STATSTG StatInfo;
		hResult = pSrcStream->Stat (&StatInfo, STATFLAG_NONAME);
		if (hResult)
		{
			TraceResult ("CXPLogon::WriteAttachsToFile: Failed to get stat info on the stream object", hResult);
			goto WriteAttachsToFileFinished;
		}

        hResult = pSrcStream->CopyTo (pDstStream,
		                              StatInfo.cbSize,
		                              NULL,
		                              NULL);
		if (hResult)
		{
			TraceResult ("CXPLogon::WriteAttachsToFile: Failed to copy attachment into destination storage", hResult);
			goto WriteAttachsToFileFinished;
		}
        // Release the streams
		pDstStream->Release();
        pDstStream = NULL;
        pSrcStream->Release();
        pSrcStream = NULL;
		// Release the attachment object since we are recycling
        // this object variable. Don't forget to NULL the parameter
        pAttach->Release();
        pAttach = NULL;
    }
    
WriteAttachsToFileFinished:
    // Clean up memory for the row set
    if (NULL != pRows)
    {
        FreeProws (pRows);
    }

    // Release used objects
    if (NULL != pDstStream)
    {
        pDstStream->Release();
    }
    if (NULL != pSrcStream)
    {
        pSrcStream->Release();
    }
    if (NULL != pAttach)
    {
        pAttach->Release();
    }
    if (NULL != pTable)
    {
        pTable->Release();
    }
    if (NULL != pRootStg)
    {
        pRootStg->Release();
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPLogon::EncodeAttachments()
//
//    Parameters
//      pTNEF       Pointer to an ITnef interface where the message
//                  attachments are going to be encoded.
//      pMsgObj     Message object that the transport is currently processing.
//
//    Purpose
//      This function encodes the transmittable attachment properties in the
//      message under submission. If the attachments IS NOT by value, then
//      the the data associated with it is encoded with TNEF. If the message
//      is by value (PR_ATTACH_METHOD == ATTACH_BY_VALUE), we exclude
//      PR_ATTACH_DATA_BIN from the list of encoded properties since it has
//      already been handled somewhere else.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPLogon::EncodeAttachments (LPITNEF pTNEF, LPMESSAGE pMsgObj)
{
    InfoTrace ("CXPLogon::EncodeAttachments method called");
    
    // Check for attachments in message
    LPMAPITABLE pTable;
    HRESULT hResult = pMsgObj->GetAttachmentTable (0, &pTable);
    if (hResult)
    {
        TraceResult ("CXPLogon::EncodeAttachments: Failed to open attachment table", hResult);
        return hResult;
    }

    LPSRowSet pRows;
    hResult = HrQueryAllRows (pTable,
                              (LPSPropTagArray)&sptAttachmentNum,
                              NULL,
                              NULL,
                              0,
                              &pRows);
    if (hResult)
    {
        TraceResult ("CXPLogon::EncodeAttachments: Failed to get the rows from the attachment table", hResult);
        pTable->Release();
        return hResult;
    }
    
    LPSPropValue pProps;
    LPSPropTagArray pTags = NULL;
    LPATTACH pAttach = NULL;
    for (ULONG i=0; i<pRows->cRows && !hResult; i++)
    {
        // For ease of coding
        pProps = pRows->aRow[i].lpProps;
        ASSERTMSG (pProps, "Where is the property array?");
        ASSERTMSG (PR_ATTACH_NUM == pProps[0].ulPropTag, "Column 0 != PR_ATTACH_NUM");
        ASSERTMSG (PR_ATTACH_METHOD == pProps[1].ulPropTag, "Column 1 != PR_ATTACH_METHOD");

        TraceString1 ("CXPLogon::EncodeAttachments: Encoding attachment number: %d", pProps[0].Value.l);
        TraceString1 ("CXPLogon::EncodeAttachments: Encoding method: %d", pProps[1].Value.l);

        // The attachment number is a "Magic" number used by the service provider to 
		// access the attachment. It is only valid in the context of
		// the current message. Do not transmit this number.
		hResult = pMsgObj->OpenAttach (pProps[0].Value.l,
                                       &IID_IMAPIProp,
                                       0,
                                       &pAttach);
		if (hResult)
		{
			TraceResult ("CXPLogon::EncodeAttachments: Failed to open attachment", hResult);
			break; // out of the FOR() loop
		}
        hResult = pAttach->GetPropList (fMapiUnicode, &pTags);
        if (hResult)
        {
            TraceResult ("CXPLogon::EncodeAttachments: Failed to get the property list", hResult);
			break; // out of the FOR() loop
        }
        // If the attachment method is BY VALUE, set the property tag to PR_NULL so that the
        // FIsTransmittable() macro excludes it from the included property list.
        // If other methods need to be handled, simply add another case testing for the
        // specific method that needs to be handled
        if (ATTACH_BY_VALUE == pProps[1].Value.l)
        {
            for (ULONG j=0; j<pTags->cValues; j++)
	        {
                if (PR_ATTACH_DATA_BIN == pTags->aulPropTag[j])
                {
                    pTags->aulPropTag[j] = PR_NULL;
                }
            }
        }
        ULONG j, cValues;
        for (cValues=0, j=0; j<pTags->cValues; j++)
	    {
		    // Use the FIsTransmittable macro in MAPI to determine if a 
		    // property is transmittable or not.
		    if (FIsTransmittable(pTags->aulPropTag[j]))
		    {
		       pTags->aulPropTag[cValues++] = pTags->aulPropTag[j];
		    }
        }
        pTags->cValues = cValues;
        hResult = pTNEF->AddProps (TNEF_PROP_ATTACHMENTS_ONLY |
                                   TNEF_PROP_INCLUDE |
                                   TNEF_PROP_CONTAINED,
                                   pProps[0].Value.l,
                                   NULL,
                                   pTags);
        gpfnFreeBuffer (pTags);
        pTags = NULL;
        pAttach->Release();
        pAttach = NULL;
    }
    // Clean up memory for the row set
    gpfnFreeBuffer (pTags);
    FreeProws (pRows);
    if (pAttach)
    {
        pAttach->Release();
    }
    pTable->Release();
    TraceMessage ("CXPLogon::EncodeAttachments: All attachments have been encoded");;
    return hResult;
}

// End of file for XPATTACH.CPP
