///////////////////////////////////////////////////////////////////////////////
//
//  File Name 
//      XPPROV.CPP
//
//  Description
//      This file implements the IXPProvider interface with the methods 
//      specified in the MAPI SPI 1.0 specifications. Also some helper
//      functions are implemented in this file.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
#define INITGUID
#define USES_IID_IXPProvider
#define USES_IID_IXPLogon
#define USES_IID_IMAPIProp
#define USES_IID_IMAPIPropData
#define USES_IID_IMAPIStatus
#define USES_IID_IMessage

#include "XPCUST.H"

#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

// Number of columns in the row of the status table for our transport
#define NUM_STATUS_ROW_PROPS    8
const static SizedSPropTagArray (NUM_STATUS_ROW_PROPS, sptStatusRow) =
{
    NUM_STATUS_ROW_PROPS,
    {
        PR_STATUS_CODE,
        PR_RESOURCE_METHODS,
        PR_PROVIDER_DISPLAY,
        PR_STATUS_STRING,
        PR_DISPLAY_NAME,
        PR_IDENTITY_DISPLAY,
        PR_IDENTITY_ENTRYID,
        PR_IDENTITY_SEARCH_KEY
    }
};

///////////////////////////////////////////////////////////////////////////////
//    CXPProvider::CXPProvider()
//
//    Parameters
//      hInst                   Handle to instance of this XP DLL
//      pMAPIMallocObj          Pointer to OLE memory allocator
//
//    Purpose
//      Constructor of the object. Parameters are passed to initialize the
//      data members with the appropiate values.
//
//    Return Value
//      None
//
CXPProvider::CXPProvider (HINSTANCE hInst, LPMALLOC pMallocObj)
{
    InfoTrace ("CXPProvider: Constructor called");
    m_hInstance      = hInst;
    m_cRef           = 1;
    m_pXPSessionList = NULL;
    m_pMallocObj     = pMallocObj;
    m_pMallocObj->AddRef();

    // Initialize critical sections for this transport
    InitializeCriticalSection (&m_csTransport);
}

///////////////////////////////////////////////////////////////////////////////
//    CXPProvider::~CXPProvider()
//
//    Parameters
//      None
//
//    Purpose
//      Close down and release resources and libraries
//
//    Return Value
//      None
//
CXPProvider::~CXPProvider()
{
    InfoTrace ("CXPProvider: Destructor called");
    m_pMallocObj->Release();
    m_pMallocObj = NULL;
    m_pXPSessionList = NULL;
    m_hInstance = NULL;

    // This is the last method called on a transport, close down the traces
    UnInitTraces();
    DeleteCriticalSection (&m_csTransport);
}

///////////////////////////////////////////////////////////////////////////////
//    CXPProvider::QueryInterface()
//
//    Parameters
//      { Refer to OLE Documentation on this method }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it 
//      returns NULL
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPProvider::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If this is one of the two IID return an interface pointer to it
    if (riid == IID_IXPProvider || riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}

///////////////////////////////////////////////////////////////////////////////
// IXPProvider virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CXPProvider::Shutdown()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      To terminate all sessions attache to this provider object
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPProvider::Shutdown (ULONG * pulFlags)
{
    InfoTrace ("CXPProvider::Shutdown method called");
    CheckParameters (IXPProvider_Shutdown, pulFlags);

    // Traverse the list of attached sessions, detach them and 
    // terminate them (including their respectives cleanups)
    CXPLogon * pLogoff = m_pXPSessionList;
    while (pLogoff)
    {
        CXPLogon * pNext = pLogoff->m_pNextSession;
        pLogoff->TransportLogoff (0);
        pLogoff->Release ();
        pLogoff = pNext;
    }
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CXPProvider::TransportLogon()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Display the logon dialog to show the options saved in the profile for
//      this provider and allow changes to it. Save new configuration settings
//      back in the profile.
//      Create a new CXPLogon object and return it to the spooler. Also, 
//      initialize the properties array for each address type handled
//      by this transport. Check all the flags and return them to the spooler
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPProvider::TransportLogon (LPMAPISUP     pSupObj,
                                          ULONG         ulUIParam,
                                          LPTSTR        pszProfileName,
                                          ULONG *       pulFlags,
                                          LPMAPIERROR * ppMAPIError,
                                          LPXPLOGON *   ppXPLogon)
{
    InfoTrace ("CXPProvider::TransportLogon method called");
    CheckParameters (IXPProvider_TransportLogon, pSupObj);

    // Open the profile for our provider and get what ever is set there
    ASSERT (pSupObj);
    LPPROFSECT pProfileObj;
    HRESULT hResult = pSupObj->OpenProfileSection (NULL, MAPI_MODIFY, &pProfileObj);
    if (hResult)
    {
        TraceResult ("CXPProvider::TransportLogon: Failed to open profile section", hResult);
        return hResult;
    }

    HANDLE hUIMutex = CreateMutex (NULL, FALSE, CONFIG_UI_MUTEX);
    if (NULL == hUIMutex)
    {
        // Local error result
        HRESULT hResult = HRESULT_FROM_WIN32 (GetLastError());
        TraceResult ("CXPProvider::TransportLogon: Failed to create UI mutext", hResult);
    }

    // Copy the property array type in the SPropValue array
    CXPLogon * pSession = NULL;
    ULONG ulPropCount;
    LPSPropValue pProps = NULL;
    hResult = pProfileObj->GetProps ((LPSPropTagArray)&sptLogonArray,
                                     fMapiUnicode,
                                     &ulPropCount,
                                     &pProps);
    if (FAILED(hResult))
    {
        TraceResult ("CXPProvider::TransportLogon: Failed to get the logon props", hResult);
        goto ErrorExit;
    }
    else
    {
        // The XPDialog is passed to our private function to display the
        // configuration page for the transport.
        XPDLG XPDialog = { 0 };
        // Fill in the logon UI structure
        XPDialog.hInst      = m_hInstance;
        XPDialog.hWnd       = (HWND)ulUIParam;
        XPDialog.ppProps    = &pProps;
        XPDialog.pMallocObj = m_pMallocObj;
        XPDialog.pSupObj    = pSupObj;
        XPDialog.hUIMutex   = hUIMutex;
        
        // In case we get MAPI_W_ERRORS_RETURNED, ignore it and reset 
        // to S_OK. Now display the logon configuration dialog
        if (MAPI_W_ERRORS_RETURNED == hResult)
        {
            if (PR_SKEL_USER_NAME != pProps[DISPLAY_NAME].ulPropTag ||
                PR_ADDRTYPE != pProps[EMAIL_ADDRESS].ulPropTag ||
                PR_EMAIL_ADDRESS != pProps[ADDRESS_TYPE].ulPropTag)
            {
                if (LOGON_NO_DIALOG & *pulFlags)
                {
                    TraceMessage ("CXPProvider::TransportLogon: UI not allowed but it is needed");
                    hResult = MAPI_E_UNCONFIGURED;
                    goto ErrorExit;
                }
                hResult = DoLogonDlg (&XPDialog);
                if (!hResult)
                {
                    hResult = pProfileObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
                }
                if (hResult)
                {
                    goto ErrorExit;
                }
            }
        }        
    }

    // Allocate the IXPLogon-derived object. Initialize its member functions 
    // with the necessary MAPI values, passed to TranportLogon()
    pSession = new CXPLogon (this,
                             pSupObj,
                             pProps,
                             hUIMutex);
    if (!pSession)
    {
        // New failed, propably due to memory shortage
        TraceMessage ("CXPProvider::TransportLogon: Failed to allocate new CXPLogon object");
        hResult = E_OUTOFMEMORY;
        goto ErrorExit;
    }
    // To avoid releasing this resources
    pProps = NULL;
    hUIMutex = NULL;
        
    // Lock other threads because we are going to work on data that is 
    // global to all threads (sessions) using this transport
    EnterCriticalSection (&m_csTransport);

    // Check the mode in which MAPI is logging into our transport and set
    // the appropiate internal (this CXPLogon object) transport session flags
    pSession->InitializeTransportStatusFlags (*pulFlags);
    
    // Initialize the transport ID prop array for this transport
    hResult = pSession->InitializePropArray();
    if (!hResult)
    {
        // Build the transport status row for our session
        hResult = InitializeStatusRow (pSession, 0);
        if (!hResult)
        {
            // Link this session object into our session list in our 
            // IXPProvider-derived (CXPProvider) object
            pSession->m_pNextSession = m_pXPSessionList ;
            m_pXPSessionList = pSession;

            // Copy our allocated object back to the returned MAPI object pointer
            *ppXPLogon = (LPXPLOGON)pSession;

            // Set the session flags returned to MAPI by the transport
            pSession->SetSessionFlags (pulFlags);
            // Release the critical section
        }
    }
    LeaveCriticalSection (&m_csTransport);

ErrorExit:
    // Release pProfileObj. This object was AddRef() by MAPI before it 
    // came to us. The transport is in charge of cleaning it up.
    if (pProfileObj)
    {
        pProfileObj->Release();
    }
    if (hUIMutex)
    {
        CloseHandle (hUIMutex);
    }
    gpfnFreeBuffer (pProps);
    if (hResult)
    {   
        // Something failed so clean the session if it has been allocated.
        // The Release() method of CXPLogon does the memory deallocation 
        // of this object. The destructor of CXPLogon does the rest of 
        // the clean up
        if (pSession)
        {
            pSession->Release();
        }
    }
    return hResult;
}                                        

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this class
//

///////////////////////////////////////////////////////////////////////////////
//    CXPProvider::InitializeStatusRow()
//
//    Parameters
//      pSession    Pointer to a CXPLogon object for which the Status 
//                  properties will be initialized
//      ulFlags     0 if the properties are being created the first time.
//                  MODIFY_FLAGS if a change is being made to the properties
//
//    Purpose
//      To initialize or modify the status properties of a CXPLogon
//      object. This function allocates an array with NUM_STATUS_ROW_PROPS
//      properties and initializes them.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CXPProvider::InitializeStatusRow (CXPLogon *    pSession,
                                               ULONG         ulFlags)
{
    // Allocate memory for the property value array of the transport's status rows.
    // ID array of property tags for the row of this transport in the session status table
    LPSPropValue pProps;
    HRESULT hResult = gpfnAllocateBuffer (sizeof (SPropValue)*NUM_STATUS_ROW_PROPS, (LPVOID *)&pProps);
    if (hResult)
    {
        TraceResult ("CXPProvider::InitializeStatusRow: Failed to allocate status row", hResult);
        return hResult;
    }
    
    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_PROVIDER_DISPLAY property: The transport readable name
    ULONG i = 0;
    pProps[i].ulPropTag = PR_PROVIDER_DISPLAY;
    pProps[i].Value.LPSZ = TRANSPORT_DISPLAY_NAME_STRING;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_RESOURCE_METHODS property. These are the methods implemented
    // in the our IMAPIStatus implementation (CMAPIStatus class.)
    pProps[i].ulPropTag = PR_RESOURCE_METHODS;
    pProps[i].Value.l = STATUS_SETTINGS_DIALOG;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_STATUS_CODE property.
    pProps[i].ulPropTag = PR_STATUS_CODE;
    pProps[i].Value.l = pSession->m_ulTransportStatus;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_STATUS_STRING property
    TCHAR szStatus[64];
    pSession->LoadStatusString (szStatus, sizeof(szStatus));
    pProps[i].ulPropTag = PR_STATUS_STRING;
    pProps[i].Value.LPSZ = szStatus;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_DISPLAY_NAME property
    pProps[i].ulPropTag = PR_DISPLAY_NAME;
    pProps[i].Value.LPSZ = pSession->m_pProps[EMAIL_ADDRESS].Value.LPSZ;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_IDENTITY_ENTRYID property
    pProps[i].ulPropTag = PR_IDENTITY_ENTRYID;
    pProps[i].Value.bin.cb = pSession->m_pIdentityProps[XPID_EID].Value.bin.cb;
    pProps[i].Value.bin.lpb = pSession->m_pIdentityProps[XPID_EID].Value.bin.lpb;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_IDENTITY_DISPLAY property
    pProps[i].ulPropTag = PR_IDENTITY_DISPLAY;
    pProps[i].Value.LPSZ = pSession->m_pIdentityProps[XPID_NAME].Value.LPSZ;
    i++;

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_IDENTITY_SEARCH_KEY property
    pProps[i].ulPropTag = PR_IDENTITY_SEARCH_KEY;
    pProps[i].Value.bin.cb = pSession->m_pIdentityProps[XPID_SEARCH_KEY].Value.bin.cb;
    pProps[i].Value.bin.lpb = pSession->m_pIdentityProps[XPID_SEARCH_KEY].Value.bin.lpb;
    i++;

    // Write the entries on the provider's session status row
    hResult = pSession->m_pSupObj->ModifyStatusRow (i, pProps, ulFlags);

    // Free the array
    gpfnFreeBuffer (pProps);
    return hResult;
}                       

// End of File for XPPROV.CPP
