///////////////////////////////////////////////////////////////////////////////
//
//  File Name
//      XPSTATUS.CPP 
//
//  Description
//      This file implements the methods for the IMAPIStatus interface.
//
//  Author
//      Irving De la Cruz
//
//  Revision: 1.0
//
// Written for Microsoft Windows Developer Support
// Copyright (c) 1994-1995 Microsoft Corporation. All rights reserved.
//
#include "XPCUST.H"

#define DO_INFO_TRACES
#ifdef DO_INFO_TRACES
#define InfoTrace(a)   TraceInfoMessage(a)
#else
#define InfoTrace(a)   
#endif // DO_INFO_TRACES

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::CMAPIStatus()
//
//    Parameters
//      pLogon          Parent logon session for this IMAPIStatus object
//      pProfSectObj    IProfSect object where the configuration properties go
//
//    Purpose
//      Constructor of the object. Parameters are passed to initialize the
//      data members with the appropiate values.
//
//    Return Value
//      None
//
CMAPIStatus::CMAPIStatus (CXPLogon * pLogon, LPPROFSECT pProfSectObj)

{
    InfoTrace ("CMAPIStatus: Constructor called");
    m_cRef          = 1;
    m_pLogon        = pLogon; // Don't AddRef() this object (to avoid circular reference)
    m_pProfSectObj  = pProfSectObj;
    if (m_pProfSectObj)
    {
        m_pProfSectObj->AddRef();
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::~CMAPIStatus()
//
//    Parameters
//      None
//
//    Purpose
//      Destructor of CMAPIStatus
//
//    Return Value
//      None
//
CMAPIStatus::~CMAPIStatus ()
{
    InfoTrace ("CMAPIStatus: Destructor called");
    if (m_pProfSectObj)
    {
        m_pProfSectObj->Release();
    }
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::QueryInterface()
//
//    Parameters
//      { Refer to OLE Documentation on this method }
//
//    Purpose
//      Returns a pointer to a interface requested if the interface is 
//      supported and implemented by this object. If it is not supported, it 
//      returns NULL
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMAPIStatus::QueryInterface (REFIID riid, LPVOID * ppvObj)
{
    // OLE requires NULLing parameter
    *ppvObj = NULL;
    // If this is one of the supported IIDs return an interface pointer to it
    if (riid == IID_IMAPIStatus || riid == IID_IMAPIProp || riid == IID_IUnknown)
    {
        *ppvObj = (LPVOID)this;
        AddRef();
        return S_OK;
    }
    // This object does not support the interface requested
    return E_NOINTERFACE;
}    

///////////////////////////////////////////////////////////////////////////////
// IMAPIProp virtual member functions
//

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::GetLastError()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by a client to get a string from the transport describing the
//      last error that occurred.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMAPIStatus::GetLastError (HRESULT         hError,
                                        ULONG           ulFlags,
                                        LPMAPIERROR *   ppMAPIError)
{
    InfoTrace ("CMAPIStatus::GetLastError method called");
    ValidateParameters (IMAPIProp_GetLastError, hError);
    // Allocate and fill a MAPIERROR structure with the error passed in.
    // Use the common helper function GetMAPIError
    return GetMAPIError (ppMAPIError, ulFlags, hError, m_pLogon->m_hInstance);
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::SaveChanges()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      S_OK always.
//
STDMETHODIMP CMAPIStatus::SaveChanges (ULONG    ulFlags)
{
    InfoTrace ("CMAPIStatus::SaveChanges method called");
    CheckParameters (IMAPIProp_SaveChanges, ulFlags);
    return S_OK;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::GetProps()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      The caller specifies an array of tags, and this method returns a
//      property value array for thoses tag, if found. The tags are the ones
//      available in the transport ID array
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMAPIStatus::GetProps (LPSPropTagArray     pTags,
                                    ULONG               ulFlags,
                                    ULONG *             pcValues,
                                    LPSPropValue *      ppPropArray)
{
    InfoTrace ("CMAPIStatus::GetProps method called");
    ValidateParameters (IMAPIProp_GetProps, pTags);
    
    #ifdef DO_INFO_TRACES
    if (!pTags)
    {
        TraceInfoMessage ("CMAPIStatus::GetProps: All properties requested");
    }
    else
    {
        TraceMessage ("CMAPIStatus::GetProps Properties requested...");
        for (ULONG j=0; j<pTags->cValues; j++) { TraceProp (pTags->aulPropTag[j]); }
    }
    #endif // DO_INFO_TRACES

    LPSPropValue pPropArray = m_pLogon->m_pProps;
	LPSPropValue pTransportIDArray = m_pLogon->m_pIdentityProps;
    BOOL fGetAllProps = pTags ? FALSE : TRUE;
	if (!pTags)
	{
        pTags = (LPSPropTagArray)&sptStatusObj;
	}
    
    *pcValues = pTags->cValues;
	HRESULT hResult = gpfnAllocateBuffer (sizeof(SPropValue)*(*pcValues), (LPVOID *)ppPropArray);
	if (hResult)
	{
        TraceResult ("CMAPIStatus::GetProps: Memory allocation failed", hResult);
		return hResult;
	}
    if (fGetAllProps)
    {
        for (ULONG i=0; i<sptStatusObj.cValues; i++)
        {
            (*ppPropArray)[i].ulPropTag = sptStatusObj.aulPropTag[i];
        }
    }
    else
    {
        // A client of the IMAPIProp::GetProps() method can pass PT_UNSPECIFIED in the
        // tag of a property. Is the provider's responsiblity to return
        // the proper type of the property.
        for (ULONG k=0; k<pTags->cValues; k++)
        {
            for (ULONG i=0; i<sptStatusObj.cValues; i++)
            {
                if (PROP_ID(pTags->aulPropTag[k]) == PROP_ID(sptStatusObj.aulPropTag[i]))
                {
                    (*ppPropArray)[k].ulPropTag = sptStatusObj.aulPropTag[i];
                }
                else
                {
                    (*ppPropArray)[k].ulPropTag = pTags->aulPropTag[k];
                }
            }
        }
    }

    TCHAR szBuffer[72];
    BOOL fGotErrors = FALSE;    
	LPVOID pSrc, pDst;
    ULONG cbSize;
    for (ULONG i=0; i<*pcValues; i++)
	{
        hResult = S_OK;
        switch ((*ppPropArray)[i].ulPropTag)
		{
            case PR_NULL :
                (*ppPropArray)[i].Value.err = S_OK;
                break;
		    case PR_RESOURCE_METHODS :
		        (*ppPropArray)[i].Value.l = STATUS_SETTINGS_DIALOG;
			    break;
            case PR_RESOURCE_FLAGS :
				hResult = E_FAIL;
                if (m_pProfSectObj)
                {
                    LPSPropValue pResFlags;
                    SPropTagArray sptResFlags = { 1, { PR_RESOURCE_FLAGS } };
                    ULONG cValues;
                    hResult = m_pProfSectObj->GetProps (&sptResFlags, fMapiUnicode, &cValues, &pResFlags);
                    if (SUCCEEDED(hResult))
                    {
                        if (PR_RESOURCE_FLAGS == pResFlags->ulPropTag)
                        {
                            (*ppPropArray)[i].Value.l = pResFlags->Value.l;
                        }
                        gpfnFreeBuffer (pResFlags);
                    }
                }
                if (hResult)
                {
				    (*ppPropArray)[i].Value.l = STATUS_PRIMARY_IDENTITY;
                    hResult = S_OK;
                }
			    break;
            case PR_RESOURCE_TYPE :
				(*ppPropArray)[i].Value.l = MAPI_TRANSPORT_PROVIDER;
			    break;
            case PR_PROVIDER_DISPLAY :
                pSrc = (LPVOID)TRANSPORT_DISPLAY_NAME_STRING;
			    break;
            case PR_PROVIDER_DLL_NAME :
                pSrc = (LPVOID)TRANSPORT_DLL_NAME_STRING;
			    break;
            case PR_CREATION_VERSION :
                (*ppPropArray)[i].Value.li.LowPart = TRANSPORT_VERSION_MAJOR;
                (*ppPropArray)[i].Value.li.HighPart = TRANSPORT_VERSION_MINOR;
                break;
		    case PR_DISPLAY_NAME :
                wsprintf (szBuffer, TEXT("%s [%s]"), pPropArray[DISPLAY_NAME].Value.LPSZ, pPropArray[EMAIL_ADDRESS].Value.LPSZ);
                pSrc = (LPVOID)szBuffer;
                break;
            // PR_SENDER_ENTRYID of the Identity properties
		    case PR_IDENTITY_ENTRYID :
                cbSize = pTransportIDArray[XPID_EID].Value.bin.cb;
                pSrc = (LPVOID)pTransportIDArray[XPID_EID].Value.bin.lpb;
            	break;
            // Email addresses of the Identity properties
		    case PR_IDENTITY_DISPLAY :
                pSrc = (LPVOID)pPropArray[DISPLAY_NAME].Value.LPSZ;
            	break;
            // PR_SENDER_SEARCH_KEY of the Identity properties
		    case PR_IDENTITY_SEARCH_KEY :
                cbSize = pTransportIDArray[XPID_SEARCH_KEY].Value.bin.cb;
                pSrc = (LPVOID)pTransportIDArray[XPID_SEARCH_KEY].Value.bin.lpb;
            	break;
		    case PR_STATUS_CODE :
				(*ppPropArray)[i].Value.l = (long)m_pLogon->m_ulTransportStatus;
			    break;
		    case PR_STATUS_STRING :
                szBuffer[0] = 0;
			    m_pLogon->LoadStatusString (szBuffer, sizeof(szBuffer) - 1);
                pSrc = (LPVOID)szBuffer;
                break;
		    case PR_OBJECT_TYPE :
				(*ppPropArray)[i].Value.l = MAPI_STATUS;
			    break;
            default:
				hResult = MAPI_E_NOT_FOUND;
                TraceRaw ("CMAPIStatus::GetProps: Unsupported property request");
                TraceProp ((*ppPropArray)[i].ulPropTag);
                break;
		}
        if (!hResult && 
            (PROP_TYPE((*ppPropArray)[i].ulPropTag) == PT_BINARY ||
             PROP_TYPE((*ppPropArray)[i].ulPropTag) == PT_TSTRING))
        {
            if (PROP_TYPE((*ppPropArray)[i].ulPropTag) == PT_TSTRING)
            {
                cbSize = Cbtszsize ((LPTSTR)pSrc);
            }
            hResult = gpfnAllocateMore (cbSize, *ppPropArray, &pDst);
            if (!hResult)
            {
                CopyMemory (pDst, pSrc, cbSize);
                if (PROP_TYPE((*ppPropArray)[i].ulPropTag) == PT_BINARY)
                {
                    (*ppPropArray)[i].Value.bin.cb  = cbSize;
                    (*ppPropArray)[i].Value.bin.lpb = (LPBYTE)pDst;
                }
                else
                {
                    (*ppPropArray)[i].Value.LPSZ = (LPTSTR)pDst;
                }
            }
        }
        if (hResult)
        {
            (*ppPropArray)[i].ulPropTag = PROP_TAG(PT_ERROR, PROP_ID((*ppPropArray)[i].ulPropTag));
            (*ppPropArray)[i].Value.err = hResult;
            fGotErrors = TRUE;
        }
	}

	if (fGotErrors)
	{
		hResult = MAPI_W_ERRORS_RETURNED;
	}
	return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::GetPropList()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Returns a tag array with the properties in the status row of this 
//      transport. The caller is responsible for freeing the memory allocated
//      for the tag array.
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMAPIStatus::GetPropList (ULONG                ulFlags,
                                       LPSPropTagArray *    ppPropTagArray)
{
    InfoTrace ("CMAPIStatus::GetPropList method called");
    ValidateParameters (IMAPIProp_GetPropList, ulFlags);

	ULONG cbTagsArraySize = CbNewSPropTagArray(NUM_STATUS_OBJECT_PROPS);
	// Allocate the required amount of memory
	HRESULT hResult = gpfnAllocateBuffer (cbTagsArraySize, (LPVOID *)ppPropTagArray);
	TraceResult ("CMAPIStatus::GetPropList: Memory allocation failed", hResult);
	if (!hResult)
    {
	    // Copy the contents of our property tag array into the buffer
	    CopyMemory (*ppPropTagArray, &sptStatusObj, cbTagsArraySize);
    }
    return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::OpenProperty()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::OpenProperty (ULONG           ulPropTag,
                                        LPCIID          piid,
                                        ULONG           ulInterfaceOptions,
                                        ULONG           ulFlags,
                                        LPUNKNOWN *     ppUnk)
{
    InfoTrace ("CMAPIStatus::OpenProperty method called");
    CheckParameters (IMAPIProp_OpenProperty, ulPropTag);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::SetProps()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::SetProps (ULONG                   cValues,
                                    LPSPropValue            pPropArray,
                                    LPSPropProblemArray *   ppProblems)
{
    InfoTrace ("CMAPIStatus::SetProps method called");
    CheckParameters (IMAPIProp_SetProps, cValues);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::DeleteProps()
//
//    Parameters
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::DeleteProps (LPSPropTagArray          pPropTagArray,
                                       LPSPropProblemArray *	ppProblems)
{
    InfoTrace ("CMAPIStatus::DeleteProps method called");
    CheckParameters (IMAPIProp_DeleteProps, pPropTagArray);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::CopyTo()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::CopyTo (ULONG                   ciidExclude,
                                  LPCIID                  rgiidExclude,
                                  LPSPropTagArray         pExcludeProps,
                                  ULONG                   ulUIParam,
                                  LPMAPIPROGRESS          pProgress,
                                  LPCIID                  pInterface,
                                  LPVOID                  pDestObj,
                                  ULONG                   ulFlags,
                                  LPSPropProblemArray *   ppProblems)
{
    InfoTrace ("CMAPIStatus::CopyTo method called");
    CheckParameters (IMAPIProp_CopyTo, ciidExclude);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::CopyProps()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::CopyProps (LPSPropTagArray            pIncludeProps,
                                     ULONG                      ulUIParam,
                                     LPMAPIPROGRESS             pProgress,
                                     LPCIID                     pInterface,
                                     LPVOID                     pDestObj,
                                     ULONG                      ulFlags,
                                     LPSPropProblemArray *      ppProblems)
{
    InfoTrace ("CMAPIStatus::CopyProps method called");
    CheckParameters (IMAPIProp_CopyProps, pIncludeProps);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::GetNamesFromIDs()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::GetNamesFromIDs (LPSPropTagArray *    ppPropTags,
                                           LPGUID               pPropSetGuid,
                                           ULONG                ulFlags,
                                           ULONG *              pcPropNames,
                                           LPMAPINAMEID **      pppPropNames)
{
    InfoTrace ("CMAPIStatus::GetNamesFromIDs method called");
    CheckParameters (IMAPIProp_GetNamesFromIDs, ppPropTags);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::GetIDsFromNames()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::GetIDsFromNames (ULONG                cPropNames,
                                           LPMAPINAMEID *       ppPropNames,
                                           ULONG                ulFlags,
                                           LPSPropTagArray *    ppPropTags)
{
    InfoTrace ("CMAPIStatus::GetIDsFromNames method called");
    CheckParameters (IMAPIProp_GetIDsFromNames, cPropNames);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
// IMAPIStatus virtual member functions implementation
//

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::ValidateState()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::ValidateState (ULONG ulUIParam, ULONG ulFlags)
{   
    InfoTrace ("CMAPIStatus::ValidateState method called");
    CheckParameters (IMAPIStatus_ValidateState, ulUIParam);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::SettingsDialog()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Called by a client to display the configuration settings, if any, for
//      a particular transport. This method could be used to display the 
//      current setting in a READ-ONLY mode
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMAPIStatus::SettingsDialog (ULONG ulUIParam, ULONG ulFlags)
{
    InfoTrace ("CMAPIStatus::SettingsDialog method called");
    ValidateParameters (IMAPIStatus_SettingsDialog, ulUIParam);

    DWORD dwWait = WaitForSingleObject (m_pLogon->m_hUIMutex, 250); // Wait for the MUTEX for 1/4 second
    if (WAIT_TIMEOUT == dwWait)
    {
        PrivateMessageBox (IDS_MSG_CONFIG_UI_IN_USE,
                           (HWND)ulUIParam,
                           MB_OK | MB_ICONSTOP,
                           m_pLogon->m_hInstance);
        return MAPI_E_BUSY;
    }

    XPDLG XPDialog = { 0 };
    HRESULT hResult;
    LPPROFSECT pProfileObj = NULL;
    LPSPropValue pTmpIdentityProps = NULL, pProps;
    // Get the Critical Section
	EnterCriticalSection (&m_pLogon->m_pXPObj->m_csTransport);

    hResult = ScDupPropset (NUM_LOGON_PROPERTIES,
                            m_pLogon->m_pProps,
                            gpfnAllocateBuffer,
                            &pProps);
	if (hResult)
    {
        TraceResult ("CMAPIStatus::SettingsDialog: Failed to duplicate logon properties", hResult);
        goto ErrorExit;
    }
    
    // Open the profile for our provider and get what ever is set there
    hResult = m_pLogon->m_pSupObj->OpenProfileSection (NULL, MAPI_MODIFY, &pProfileObj);
    if (hResult)
    {
        TraceResult ("CMAPIStatus::SettingsDialog: Failed to open the profile section", hResult);
        goto ErrorExit;
    }

    // Fill in the logon UI structure
    XPDialog.hInst      = m_pLogon->m_hInstance;
	XPDialog.hWnd       = (HWND)ulUIParam;
	XPDialog.ppProps    = &pProps;
	XPDialog.pMallocObj = m_pLogon->m_pXPObj->m_pMallocObj;
	XPDialog.pSupObj    = m_pLogon->m_pSupObj;
	XPDialog.ulFlags	= ulFlags;
    
    hResult = DoLogonDlg (&XPDialog);
    if (hResult)
    {
        goto ErrorExit;
    }
    
    // If the dialog was displayed READ-only, we don't need to verify
    // anything because the user didn't change anything and we are
    // still runnig with the logon setting of the session.
    if (ulFlags & UI_READONLY)
    {
        goto ErrorExit;
    }

    // We don't need to set the header file name back into the profile.
    hResult = pProfileObj->SetProps (NUM_LOGON_PROPERTIES, pProps, NULL);
    if (hResult)
    {
        TraceResult ("CMAPIStatus::SetttingsDialog: Failed to save the properties back into the profile", hResult);
    }

	// Initialize the new Transport ID property array
    hResult = InitializePropArray (&pTmpIdentityProps);
    if (hResult)
    {
        goto ErrorExit;
    }

	// Replace the original PropArray with the new one.
	gpfnFreeBuffer (m_pLogon->m_pProps);
	m_pLogon->m_pProps = pProps;
 
    pProps = NULL; // To avoid releasing this array down below.

	// Replace the original Transport ID prop array with the new one.
	gpfnFreeBuffer (m_pLogon->m_pIdentityProps);
    m_pLogon->m_pIdentityProps = pTmpIdentityProps;
    pTmpIdentityProps = NULL; // To avoid releasing this array down below.

	hResult = m_pLogon->m_pXPObj->InitializeStatusRow (m_pLogon, STATUSROW_UPDATE);

ErrorExit:
	// Release the Critical Section.
	LeaveCriticalSection (&m_pLogon->m_pXPObj->m_csTransport);
	if (pProfileObj)
    {
        pProfileObj->Release();
	}
	gpfnFreeBuffer (pProps);
	gpfnFreeBuffer (pTmpIdentityProps);
    ReleaseMutex (m_pLogon->m_hUIMutex);
	return hResult;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::ChangePassword()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Stub method.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::ChangePassword (LPTSTR    pszOldPass,
                                          LPTSTR    pszNewPass,
                                          ULONG     ulFlags)
{
    InfoTrace ("CMAPIStatus::ChangePassword method called");
    CheckParameters (IMAPIStatus_ChangePassword, pszOldPass);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::FlushQueues()
//
//    Parameters
//      { Refer to MAPI Documentation on this method }
//
//    Purpose
//      Place holder. Replace this function with actual implementation 
//      if needed.
//
//    Return Value
//      MAPI_E_NO_SUPPORT always.
//
STDMETHODIMP CMAPIStatus::FlushQueues (ULONG        ulUIParam,
                                       ULONG        cbTargetTransport,
                                       LPENTRYID	pTargetTransport,
                                       ULONG        ulFlags)
{
    InfoTrace ("CMAPIStatus::FlushQueues method called");
    CheckParameters (IMAPIStatus_ValidateState, ulUIParam);
    return MAPI_E_NO_SUPPORT;
}

///////////////////////////////////////////////////////////////////////////////
// Other member functions specific to this transport
//

///////////////////////////////////////////////////////////////////////////////
//    CMAPIStatus::InitializePropArray()
//
//    Parameters
//      ppPropValue     Pointer to a pointer where this method return an 
//                      newly allocated property value array
//
//    Purpose
//      Returns a property array with the entry of the user of this transport.
//      The caller is responsible for cleaning the allocated memory in this
//      method
//
//    Return Value
//      An HRESULT
//
STDMETHODIMP CMAPIStatus::InitializePropArray (LPSPropValue * ppPropValue)
{
    InfoTrace ("CMAPIStatus::InitializePropArray method called");
    
    // If not allocated yet, allocate the initial property array for 
    // transport ID. This memory block is freed in the destructor of
    // the CXPLogon object
    HRESULT hResult;
    if (NULL == *ppPropValue)
    {
        hResult = gpfnAllocateBuffer (sizeof(SPropValue)*NUM_SENDER_PROPS, (LPVOID *)ppPropValue);
        if (hResult)
        {
            TraceResult ("CMAPIStatus::InitializePropArray: Memory allocation failed", hResult);
            return hResult;
        }
    }
    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_SENDER_ENTRYID property
    // Create an entry directly in the property array
    (*ppPropValue)[0].ulPropTag = PR_SENDER_ENTRYID;
    hResult = m_pLogon->m_pSupObj->CreateOneOff (m_pLogon->m_pProps[DISPLAY_NAME].Value.LPSZ,
                                                 m_pLogon->m_pProps[ADDRESS_TYPE].Value.LPSZ,
                                                 m_pLogon->m_pProps[EMAIL_ADDRESS].Value.LPSZ,
                                                 fMapiUnicode,
                                                 &(*ppPropValue)[0].Value.bin.cb,
                                                 (LPENTRYID *)&(*ppPropValue)[0].Value.bin.lpb);
    if (hResult)
    {
        TraceResult ("CMAPIStatus::InitializePropArray: Failed to create a one off identity address", hResult);
        (*ppPropValue)[0].Value.bin.lpb = NULL;
        return hResult;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_SENDER_NAME property
    (*ppPropValue)[1].ulPropTag = PR_SENDER_NAME;
    LPTSTR pStr1, pStr2 = m_pLogon->m_pProps[DISPLAY_NAME].Value.LPSZ;
    hResult = gpfnAllocateMore (Cbtszsize(pStr2), *ppPropValue, (LPVOID *)&pStr1);
    if (hResult)
    {
        TraceResult ("CMAPIStatus::InitializePropArray: Failed to allocate sender name", hResult);
        return hResult;
    }
    lstrcpy (pStr1, pStr2);
    (*ppPropValue)[1].Value.LPSZ = pStr1;
    
    ///////////////////////////////////////////////////////////////////////////
    // Set the PR_SENDER_SEARCH_KEY property
    (*ppPropValue)[2].ulPropTag = PR_SENDER_SEARCH_KEY;

    // PR_SENDER_SEARCH_KEY is "TYPE:ADDRESS". Uppercase is required
    // The size of the property is the size of the string char type plus a 
    // colon plus the size address string (and address type string) plus a 
    // NULL char
    ULONG cbStrSize = Cbtszsize (m_pLogon->m_pProps[ADDRESS_TYPE].Value.LPSZ) +
                      Cbtszsize (":") +
                      Cbtszsize (m_pLogon->m_pProps[EMAIL_ADDRESS].Value.LPSZ) +
                      sizeof(TCHAR); // NULL terminator
    hResult = gpfnAllocateMore (cbStrSize, *ppPropValue, (LPVOID *)&pStr1);
    if (hResult)
    {
        TraceResult ("CMAPIStatus::InitializePropArray: Failed to allocate sender search key", hResult);
        return hResult;
    }
    wsprintf (pStr1, TEXT("%s:%s"), m_pLogon->m_pProps[ADDRESS_TYPE].Value.LPSZ, m_pLogon->m_pProps[EMAIL_ADDRESS].Value.LPSZ);
    CharUpperBuff (pStr1, (DWORD)lstrlen(pStr1));
    (*ppPropValue)[2].Value.bin.cb  = cbStrSize;
    (*ppPropValue)[2].Value.bin.lpb = (LPBYTE)pStr1;
    return hResult;
}

// End of File for XPSTATUS.CPP
